@@ -19,35 +19,43 @@
 # <pep8 compliant>
 import bpy
 import nodeitems_utils
-from nodeitems_utils import NodeCategory, NodeItem, NodeItemCustom
+from nodeitems_utils import (
+    NodeCategory,
+    NodeItem,
+    NodeItemCustom,
+)
 
 
 # Subclasses for standard node types
 
-class CompositorNodeCategory(NodeCategory):
+class SortedNodeCategory(NodeCategory):
+    def __init__(self, identifier, name, description="", items=None):
+        # for builtin nodes the convention is to sort by name
+        if isinstance(items, list):
+            items = sorted(items, key=lambda item: item.label.lower())
+
+        super().__init__(identifier, name, description, items)
+
+
+class CompositorNodeCategory(SortedNodeCategory):
     @classmethod
     def poll(cls, context):
-        return (context.space_data.tree_type == 'CompositorNodeTree')
+        return (context.space_data.type == 'NODE_EDITOR' and
+                context.space_data.tree_type == 'CompositorNodeTree')
 
 
-class ShaderNewNodeCategory(NodeCategory):
+class ShaderNodeCategory(SortedNodeCategory):
     @classmethod
     def poll(cls, context):
-        return (context.space_data.tree_type == 'ShaderNodeTree' and
-                context.scene.render.use_shading_nodes)
+        return (context.space_data.type == 'NODE_EDITOR' and
+                context.space_data.tree_type == 'ShaderNodeTree')
 
 
-class ShaderOldNodeCategory(NodeCategory):
+class TextureNodeCategory(SortedNodeCategory):
     @classmethod
     def poll(cls, context):
-        return (context.space_data.tree_type == 'ShaderNodeTree' and
-                not context.scene.render.use_shading_nodes)
-
-
-class TextureNodeCategory(NodeCategory):
-    @classmethod
-    def poll(cls, context):
-        return context.space_data.tree_type == 'TextureNodeTree'
+        return (context.space_data.type == 'NODE_EDITOR' and
+                context.space_data.tree_type == 'TextureNodeTree')
 
 
 # menu entry for node group tools
@@ -56,12 +64,13 @@ def group_tools_draw(self, layout, context):
     layout.operator("node.group_ungroup")
     layout.separator()
 
+
 # maps node tree type to group node type
 node_tree_group_type = {
     'CompositorNodeTree': 'CompositorNodeGroup',
     'ShaderNodeTree': 'ShaderNodeGroup',
     'TextureNodeTree': 'TextureNodeGroup',
-    }
+}
 
 
 # generic node group items generator for shader, compositor and texture node groups
@@ -93,7 +102,9 @@ def node_group_items(context):
         # filter out recursive groups
         if contains_group(group, ntree):
             continue
-
+        # filter out hidden nodetrees
+        if group.name.startswith('.'):
+            continue
         yield NodeItem(node_tree_group_type[group.bl_idname],
                        group.name,
                        {"node_tree": "bpy.data.node_groups[%r]" % group.name})
@@ -128,58 +139,39 @@ def object_shader_nodes_poll(context):
             snode.shader_type == 'OBJECT')
 
 
+def cycles_shader_nodes_poll(context):
+    return context.engine == 'CYCLES'
+
+
+def eevee_shader_nodes_poll(context):
+    return context.engine == 'BLENDER_EEVEE'
+
+
+def eevee_cycles_shader_nodes_poll(context):
+    return (cycles_shader_nodes_poll(context) or
+            eevee_shader_nodes_poll(context))
+
+
+def object_cycles_shader_nodes_poll(context):
+    return (object_shader_nodes_poll(context) and
+            cycles_shader_nodes_poll(context))
+
+
+def object_eevee_shader_nodes_poll(context):
+    return (object_shader_nodes_poll(context) and
+            eevee_shader_nodes_poll(context))
+
+
+def object_eevee_cycles_shader_nodes_poll(context):
+    return (object_shader_nodes_poll(context) and
+            eevee_cycles_shader_nodes_poll(context))
+
+
 # All standard node categories currently used in nodes.
 
 shader_node_categories = [
-    # Shader Nodes
-    ShaderOldNodeCategory("SH_INPUT", "Input", items=[
-        NodeItem("ShaderNodeMaterial"),
-        NodeItem("ShaderNodeCameraData"),
-        NodeItem("ShaderNodeLampData"),
-        NodeItem("ShaderNodeValue"),
-        NodeItem("ShaderNodeRGB"),
-        NodeItem("ShaderNodeTexture"),
-        NodeItem("ShaderNodeGeometry"),
-        NodeItem("ShaderNodeExtendedMaterial"),
-        NodeItem("ShaderNodeParticleInfo"),
-        NodeItem("NodeGroupInput", poll=group_input_output_item_poll),
-        ]),
-    ShaderOldNodeCategory("SH_OUTPUT", "Output", items=[
-        NodeItem("ShaderNodeOutput"),
-        NodeItem("NodeGroupOutput", poll=group_input_output_item_poll),
-        ]),
-    ShaderOldNodeCategory("SH_OP_COLOR", "Color", items=[
-        NodeItem("ShaderNodeMixRGB"),
-        NodeItem("ShaderNodeRGBCurve"),
-        NodeItem("ShaderNodeInvert"),
-        NodeItem("ShaderNodeHueSaturation"),
-        NodeItem("ShaderNodeGamma"),
-        ]),
-    ShaderOldNodeCategory("SH_OP_VECTOR", "Vector", items=[
-        NodeItem("ShaderNodeNormal"),
-        NodeItem("ShaderNodeMapping"),
-        NodeItem("ShaderNodeVectorCurve"),
-        NodeItem("ShaderNodeVectorTransform"),
-        ]),
-    ShaderOldNodeCategory("SH_CONVERTOR", "Converter", items=[
-        NodeItem("ShaderNodeValToRGB"),
-        NodeItem("ShaderNodeRGBToBW"),
-        NodeItem("ShaderNodeMath"),
-        NodeItem("ShaderNodeVectorMath"),
-        NodeItem("ShaderNodeSqueeze"),
-        NodeItem("ShaderNodeSeparateRGB"),
-        NodeItem("ShaderNodeCombineRGB"),
-        NodeItem("ShaderNodeSeparateHSV"),
-        NodeItem("ShaderNodeCombineHSV"),
-        ]),
-    ShaderOldNodeCategory("SH_GROUP", "Group", items=node_group_items),
-    ShaderOldNodeCategory("SH_LAYOUT", "Layout", items=[
-        NodeItem("NodeFrame"),
-        NodeItem("NodeReroute"),
-        ]),
-
-    # New Shader Nodes (Cycles)
-    ShaderNewNodeCategory("SH_NEW_INPUT", "Input", items=[
+    # Shader Nodes (Cycles and Eevee)
+    ShaderNodeCategory("SH_NEW_INPUT", "Input", items=[
         NodeItem("ShaderNodeTexCoord"),
         NodeItem("ShaderNodeAttribute"),
         NodeItem("ShaderNodeLightPath"),
@@ -190,6 +182,8 @@ shader_node_categories = [
         NodeItem("ShaderNodeTangent"),
         NodeItem("ShaderNodeNewGeometry"),
         NodeItem("ShaderNodeWireframe"),
+        NodeItem("ShaderNodeBevel"),
+        NodeItem("ShaderNodeAmbientOcclusion"),
         NodeItem("ShaderNodeObjectInfo"),
         NodeItem("ShaderNodeHairInfo"),
         NodeItem("ShaderNodeParticleInfo"),
@@ -197,36 +191,39 @@ shader_node_categories = [
         NodeItem("ShaderNodeUVMap"),
         NodeItem("ShaderNodeUVAlongStroke", poll=line_style_shader_nodes_poll),
         NodeItem("NodeGroupInput", poll=group_input_output_item_poll),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_OUTPUT", "Output", items=[
-        NodeItem("ShaderNodeOutputMaterial", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeOutputLamp", poll=object_shader_nodes_poll),
+    ]),
+    ShaderNodeCategory("SH_NEW_OUTPUT", "Output", items=[
+        NodeItem("ShaderNodeOutputMaterial", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeOutputLight", poll=object_cycles_shader_nodes_poll),
         NodeItem("ShaderNodeOutputWorld", poll=world_shader_nodes_poll),
         NodeItem("ShaderNodeOutputLineStyle", poll=line_style_shader_nodes_poll),
         NodeItem("NodeGroupOutput", poll=group_input_output_item_poll),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_SHADER", "Shader", items=[
-        NodeItem("ShaderNodeMixShader"),
-        NodeItem("ShaderNodeAddShader"),
-        NodeItem("ShaderNodeBsdfDiffuse", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfGlossy", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfTransparent", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfRefraction", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfGlass", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfTranslucent", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfAnisotropic", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfVelvet", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfToon", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeSubsurfaceScattering", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeEmission", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeBsdfHair", poll=object_shader_nodes_poll),
+    ]),
+    ShaderNodeCategory("SH_NEW_SHADER", "Shader", items=[
+        NodeItem("ShaderNodeMixShader", poll=eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeAddShader", poll=eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfDiffuse", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfPrincipled", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfGlossy", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfTransparent", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfRefraction", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfGlass", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfTranslucent", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfAnisotropic", poll=object_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfVelvet", poll=object_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfToon", poll=object_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeSubsurfaceScattering", poll=object_eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeEmission", poll=eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfHair", poll=object_cycles_shader_nodes_poll),
         NodeItem("ShaderNodeBackground", poll=world_shader_nodes_poll),
-        NodeItem("ShaderNodeAmbientOcclusion", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeHoldout", poll=object_shader_nodes_poll),
-        NodeItem("ShaderNodeVolumeAbsorption"),
-        NodeItem("ShaderNodeVolumeScatter"),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_TEXTURE", "Texture", items=[
+        NodeItem("ShaderNodeHoldout", poll=object_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeVolumeAbsorption", poll=eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeVolumeScatter", poll=eevee_cycles_shader_nodes_poll),
+        NodeItem("ShaderNodeVolumePrincipled"),
+        NodeItem("ShaderNodeEeveeSpecular", poll=object_eevee_shader_nodes_poll),
+        NodeItem("ShaderNodeBsdfHairPrincipled", poll=object_cycles_shader_nodes_poll)
+    ]),
+    ShaderNodeCategory("SH_NEW_TEXTURE", "Texture", items=[
         NodeItem("ShaderNodeTexImage"),
         NodeItem("ShaderNodeTexEnvironment"),
         NodeItem("ShaderNodeTexSky"),
@@ -239,8 +236,9 @@ shader_node_categories = [
         NodeItem("ShaderNodeTexChecker"),
         NodeItem("ShaderNodeTexBrick"),
         NodeItem("ShaderNodeTexPointDensity"),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_OP_COLOR", "Color", items=[
+        NodeItem("ShaderNodeTexIES"),
+    ]),
+    ShaderNodeCategory("SH_NEW_OP_COLOR", "Color", items=[
         NodeItem("ShaderNodeMixRGB"),
         NodeItem("ShaderNodeRGBCurve"),
         NodeItem("ShaderNodeInvert"),
@@ -248,19 +246,22 @@ shader_node_categories = [
         NodeItem("ShaderNodeHueSaturation"),
         NodeItem("ShaderNodeGamma"),
         NodeItem("ShaderNodeBrightContrast"),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_OP_VECTOR", "Vector", items=[
+    ]),
+    ShaderNodeCategory("SH_NEW_OP_VECTOR", "Vector", items=[
         NodeItem("ShaderNodeMapping"),
         NodeItem("ShaderNodeBump"),
+        NodeItem("ShaderNodeDisplacement"),
+        NodeItem("ShaderNodeVectorDisplacement"),
         NodeItem("ShaderNodeNormalMap"),
         NodeItem("ShaderNodeNormal"),
         NodeItem("ShaderNodeVectorCurve"),
         NodeItem("ShaderNodeVectorTransform"),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_CONVERTOR", "Converter", items=[
+    ]),
+    ShaderNodeCategory("SH_NEW_CONVERTOR", "Converter", items=[
         NodeItem("ShaderNodeMath"),
         NodeItem("ShaderNodeValToRGB"),
         NodeItem("ShaderNodeRGBToBW"),
+        NodeItem("ShaderNodeShaderToRGB", poll=object_eevee_shader_nodes_poll),
         NodeItem("ShaderNodeVectorMath"),
         NodeItem("ShaderNodeSeparateRGB"),
         NodeItem("ShaderNodeCombineRGB"),
@@ -270,16 +271,16 @@ shader_node_categories = [
         NodeItem("ShaderNodeCombineHSV"),
         NodeItem("ShaderNodeWavelength"),
         NodeItem("ShaderNodeBlackbody"),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_SCRIPT", "Script", items=[
+    ]),
+    ShaderNodeCategory("SH_NEW_SCRIPT", "Script", items=[
         NodeItem("ShaderNodeScript"),
-        ]),
-    ShaderNewNodeCategory("SH_NEW_GROUP", "Group", items=node_group_items),
-    ShaderNewNodeCategory("SH_NEW_LAYOUT", "Layout", items=[
+    ]),
+    ShaderNodeCategory("SH_NEW_GROUP", "Group", items=node_group_items),
+    ShaderNodeCategory("SH_NEW_LAYOUT", "Layout", items=[
         NodeItem("NodeFrame"),
         NodeItem("NodeReroute"),
-        ]),
-    ]
+    ]),
+]
 
 compositor_node_categories = [
     # Compositor Nodes
@@ -295,7 +296,7 @@ compositor_node_categories = [
         NodeItem("CompositorNodeTime"),
         NodeItem("CompositorNodeTrackPos"),
         NodeItem("NodeGroupInput", poll=group_input_output_item_poll),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_OUTPUT", "Output", items=[
         NodeItem("CompositorNodeComposite"),
         NodeItem("CompositorNodeViewer"),
@@ -303,7 +304,7 @@ compositor_node_categories = [
         NodeItem("CompositorNodeOutputFile"),
         NodeItem("CompositorNodeLevels"),
         NodeItem("NodeGroupOutput", poll=group_input_output_item_poll),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_OP_COLOR", "Color", items=[
         NodeItem("CompositorNodeMixRGB"),
         NodeItem("CompositorNodeAlphaOver"),
@@ -317,7 +318,7 @@ compositor_node_categories = [
         NodeItem("CompositorNodeColorCorrection"),
         NodeItem("CompositorNodeTonemap"),
         NodeItem("CompositorNodeZcombine"),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_CONVERTOR", "Converter", items=[
         NodeItem("CompositorNodeMath"),
         NodeItem("CompositorNodeValToRGB"),
@@ -334,7 +335,7 @@ compositor_node_categories = [
         NodeItem("CompositorNodeSepYCCA"),
         NodeItem("CompositorNodeCombYCCA"),
         NodeItem("CompositorNodeSwitchView"),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_OP_FILTER", "Filter", items=[
         NodeItem("CompositorNodeBlur"),
         NodeItem("CompositorNodeBilateralblur"),
@@ -349,14 +350,14 @@ compositor_node_categories = [
         NodeItem("CompositorNodeDBlur"),
         NodeItem("CompositorNodePixelate"),
         NodeItem("CompositorNodeSunBeams"),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_OP_VECTOR", "Vector", items=[
         NodeItem("CompositorNodeNormal"),
         NodeItem("CompositorNodeMapValue"),
         NodeItem("CompositorNodeMapRange"),
         NodeItem("CompositorNodeNormalize"),
         NodeItem("CompositorNodeCurveVec"),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_MATTE", "Matte", items=[
         NodeItem("CompositorNodeKeying"),
         NodeItem("CompositorNodeKeyingScreen"),
@@ -370,7 +371,8 @@ compositor_node_categories = [
         NodeItem("CompositorNodeChromaMatte"),
         NodeItem("CompositorNodeColorMatte"),
         NodeItem("CompositorNodeDoubleEdgeMask"),
-        ]),
+        NodeItem("CompositorNodeCryptomatte"),
+    ]),
     CompositorNodeCategory("CMP_DISTORT", "Distort", items=[
         NodeItem("CompositorNodeScale"),
         NodeItem("CompositorNodeLensdist"),
@@ -385,14 +387,14 @@ compositor_node_categories = [
         NodeItem("CompositorNodeStabilize"),
         NodeItem("CompositorNodePlaneTrackDeform"),
         NodeItem("CompositorNodeCornerPin"),
-        ]),
+    ]),
     CompositorNodeCategory("CMP_GROUP", "Group", items=node_group_items),
     CompositorNodeCategory("CMP_LAYOUT", "Layout", items=[
         NodeItem("NodeFrame"),
         NodeItem("NodeReroute"),
         NodeItem("CompositorNodeSwitch"),
-        ]),
-    ]
+    ]),
+]
 
 texture_node_categories = [
     # Texture Nodes
@@ -402,12 +404,12 @@ texture_node_categories = [
         NodeItem("TextureNodeTexture"),
         NodeItem("TextureNodeImage"),
         NodeItem("NodeGroupInput", poll=group_input_output_item_poll),
-        ]),
+    ]),
     TextureNodeCategory("TEX_OUTPUT", "Output", items=[
         NodeItem("TextureNodeOutput"),
         NodeItem("TextureNodeViewer"),
         NodeItem("NodeGroupOutput", poll=group_input_output_item_poll),
-        ]),
+    ]),
     TextureNodeCategory("TEX_OP_COLOR", "Color", items=[
         NodeItem("TextureNodeMixRGB"),
         NodeItem("TextureNodeCurveRGB"),
@@ -415,11 +417,11 @@ texture_node_categories = [
         NodeItem("TextureNodeHueSaturation"),
         NodeItem("TextureNodeCompose"),
         NodeItem("TextureNodeDecompose"),
-        ]),
+    ]),
     TextureNodeCategory("TEX_PATTERN", "Pattern", items=[
         NodeItem("TextureNodeChecker"),
         NodeItem("TextureNodeBricks"),
-        ]),
+    ]),
     TextureNodeCategory("TEX_TEXTURE", "Textures", items=[
         NodeItem("TextureNodeTexNoise"),
         NodeItem("TextureNodeTexDistNoise"),
@@ -431,26 +433,26 @@ texture_node_categories = [
         NodeItem("TextureNodeTexWood"),
         NodeItem("TextureNodeTexMusgrave"),
         NodeItem("TextureNodeTexStucci"),
-        ]),
+    ]),
     TextureNodeCategory("TEX_CONVERTOR", "Converter", items=[
         NodeItem("TextureNodeMath"),
         NodeItem("TextureNodeValToRGB"),
         NodeItem("TextureNodeRGBToBW"),
         NodeItem("TextureNodeValToNor"),
         NodeItem("TextureNodeDistance"),
-        ]),
+    ]),
     TextureNodeCategory("TEX_DISTORT", "Distort", items=[
         NodeItem("TextureNodeScale"),
         NodeItem("TextureNodeTranslate"),
         NodeItem("TextureNodeRotate"),
         NodeItem("TextureNodeAt"),
-        ]),
+    ]),
     TextureNodeCategory("TEX_GROUP", "Group", items=node_group_items),
     TextureNodeCategory("TEX_LAYOUT", "Layout", items=[
         NodeItem("NodeFrame"),
         NodeItem("NodeReroute"),
-        ]),
-    ]
+    ]),
+]
 
 
 def register():
